/* @alwatr/async-queue v1.3.8 */
"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/main.ts
var main_exports = {};
__export(main_exports, {
  AsyncQueue: () => AsyncQueue
});
module.exports = __toCommonJS(main_exports);
var import_flatomise = require("@alwatr/flatomise");
var import_package_tracer = require("@alwatr/package-tracer");
__dev_mode__: import_package_tracer.packageTracer.add("@alwatr/async-queue", "1.3.8");
var AsyncQueue = class {
  constructor() {
    /**
     * A record of task IDs and their corresponding last queued task promises.
     */
    this.queue__ = {};
  }
  /**
   * Push a async task to the queue.
   *
   * @param taskId task id
   * @param task async task
   * @returns A promise that resolves when the task is done.
   *
   * @example
   * ```typescript
   * const queue = new AsyncQueue();
   *
   * function longTask() {
   *  queue.push('longTaskId', async () => {
   *   // ...
   * });
   * ```
   */
  async push(taskId, task) {
    const flatomise = (0, import_flatomise.newFlatomise)();
    const previousTaskPromise = this.queue__[taskId];
    this.queue__[taskId] = flatomise.promise;
    try {
      await previousTaskPromise;
    } catch (_e) {
    }
    setTimeout(() => {
      task().then(flatomise.resolve, flatomise.reject).then(() => {
        if (this.queue__[taskId] === flatomise.promise) {
          delete this.queue__[taskId];
        }
      });
    }, 0);
    return flatomise.promise;
  }
  /**
   * Check if the task running in the queue.
   *
   * @param taskId task id
   * @returns true if the task is running, otherwise false.
   * @example
   * ```typescript
   * if (queue.isRunning('longTaskId')) {
   *  // ...
   * }
   * ```
   */
  isRunning(taskId) {
    return this.queue__[taskId] !== void 0;
  }
  /**
   * Wait for the all tasks in the queue to finish.
   *
   * @param taskId task id
   * @returns A promise that resolves when all tasks are done.
   * @example
   * ```typescript
   * await queue.waitForFinish('longTaskId');
   * ```
   */
  waitForFinish(taskId) {
    return this.queue__[taskId] ?? Promise.resolve();
  }
  /**
   * Wait for the all tasks in the queue to finish.
   * @returns A promise that resolves when all tasks are done.
   * @example
   * ```typescript
   * await queue.waitForAllFinish();
   * ```
   */
  waitForAllFinish() {
    return Promise.all(Object.values(this.queue__));
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  AsyncQueue
});
/*! For license information please see main.cjs.LEGAL.txt */
//# sourceMappingURL=main.cjs.map
